//+------------------------------------------------------------------+
//|                                               HistoryDumpCSV.mq4 |
//|                                       Copyright 2015, Mark Flint |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Mark Flint"
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
/**********
This script creates a .CSV file of your order history so you can import it into Excel and analyse it.

It extracts date / time information in both native MetaTrader format ( _MT ) and Excel format.
It also supports aggregate analysis by calculating a 'Heat map' day and time slot.
The heat map default is 4 slots per hour, i.e. 15 minutes.  This means trades within that 
same slot will have the same slot number.  This can make your Excel analysis and creation of 
useful pivot tables a bit easier.
The script will also FTP the file to your specified FTP server, if you have defined one in your terminal.
Beware that if you have filtered your account history to, say, the last 3 months, then this is 
all the data that will be extracted. If you want ALL your history then remove your history filter. 

***********/
//+------------------------------------------------------------------+
//| structure of history order entry                                 |
//+------------------------------------------------------------------+
struct Orders
  {
   double            Point; // not output
   int               AccNumber;
   int               SequenceNumber;
   long              Ticket; // ID
   string            Type; // OP_BUY, OP_SELLSTOP etc.  
   string            Lots; // double  
   string            Symbol; 
   datetime          ServerOpenTime_MT;
   //string          ServerOpenTime_T;   
   string            A1;
   datetime          ServerCloseTime_MT;
   //string          ServerCloseTime_T; 
   string            A2;
   string            A3;   
   string            OpenPrice; // double             
   string            ClosePrice; //double
   string            Pips;//double 
   string            A4;      
   string            Profit; //double // without swaps and commissions 
   string            PriceMove;       
   string            Commission; //double
   //string          ServerExpiration_T;
   string            Swap;//double   
   int               MagicNumber;
   string            Comment;  
   string            A5;    
   string            A6;           
   string            StopLoss;//double
   string            TakeProfit;//double
  };

string AccountID=IntegerToString(AccountNumber());
string reportfilename=StringConcatenate("HistDump_Acc_",AccountID,".CSV");  // the FileOpen has the FILE_COMMON flag meaning it is saved to the Commong/Files folder.

int filehandle;
bool NeedToOpenFile=true; // Set this to allow the process to open file and write the header.
int SequenceNum=0;

bool SendFileByFTP=true; // If not needed, set this to false.

string msg="";
MqlDateTime ServerMqlDateTimeGMT;
long ServerTimeGMTOffsetSeconds=0;
int SLOTSPERHOUR=4;  // This divides the trading time into 15 minute slots for aggregating data.  2=30 minutes, 3=20 minutes, 1=1hour slots.
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//---
   msg=StringConcatenate("HistoryDumpCSV: Save history data to ",reportfilename);
   MessageBox(msg);
   int i;
   bool FoundOrder,SavedDataOK;
   int total=OrdersHistoryTotal();
   Orders ThisOrder;
   ServerTimeGMTOffsetSeconds=TimeGMT()-TimeCurrent(); // negative if behind GMT, positive if ahead of GMT.

   for(i=0; i<=total; i++)
     {
      FoundOrder=OrderSelect(i,SELECT_BY_POS,MODE_HISTORY);
      FillOrderData(ThisOrder);
      if(!FoundOrder || ThisOrder.Ticket<1) continue;
      SavedDataOK=SaveData(ThisOrder);
      if(!SavedDataOK) break; // error, exit.
     };
   if(!NeedToOpenFile) // file was openned, so now close it
     {
      FileFlush(filehandle); // make sure it is flushed to disk before we exit
      FileClose(filehandle);
      if(SendFileByFTP)
        {
         if(!SendFTP(reportfilename))
           {
            PlaySound("alert.wav");
            msg=StringConcatenate("HistoryDumpCSV: ERROR: FTP Failed to send file ",reportfilename);
            Print(msg); // log error in Journal
            MessageBox(msg); // interrupt user
            return;
           }
         else
           {
            msg="HistoryDumpCSV: File generated and successfully sent by FTP.";
            Print(msg); // log to Journal
            PlaySound("ok.wav"); // file generated and successfully sent by FTP.  OK!
           }
        }
      else
        {
         msg="HistoryDumpCSV: File generated successfully.  Not sent by FTP as per settings.";
         Print(msg); // log to Journal
         PlaySound("ok.wav"); // file generated successfully. FTP not wanted. OK!
        }
     }
   else
     {
      PlaySound("alert.wav");
      msg="HistoryDumpCSV: File open error / No file generated.";
      Print(msg); // log to Journal
      MessageBox(msg); // interrupt user
     }
   return;
  }
//+------------------------------------------------------------------+
//|  Fill Orders structure object                                    |
//+------------------------------------------------------------------+
void FillOrderData(Orders &ThisOrder)
  {
// Use immediately after OrderSelect, OrderSend
   ThisOrder.AccNumber=AccountNumber();
   ThisOrder.SequenceNumber=++SequenceNum;   
   ThisOrder.Ticket=OrderTicket(); // ID
   ThisOrder.Type=OrderTypeName(OrderType()); // string name from int for OP_BUY, OP_SELLSTOP etc.    
   ThisOrder.Lots=DoubleToString(OrderLots(),2); 
   ThisOrder.Symbol=OrderSymbol();
   ThisOrder.ServerOpenTime_MT=OrderOpenTime();      
   ThisOrder.Point=SymbolInfoDouble(OrderSymbol(),SYMBOL_POINT); 
   ThisOrder.A1="";  
   ThisOrder.ServerCloseTime_MT=OrderCloseTime(); // native MT  
   ThisOrder.A2=""; 
   ThisOrder.A3="";    
   ThisOrder.OpenPrice=DoubleToString(OrderOpenPrice(),Digits);              
   ThisOrder.ClosePrice=DoubleToString(OrderClosePrice(),Digits);
   ThisOrder.Pips="0.0";
   if(ThisOrder.Type=="BUY")  ThisOrder.Pips=DoubleToString(NormalizeDouble(PointsToPips(PriceMoveToPoints(OrderClosePrice()-OrderOpenPrice(),ThisOrder.Point)),Digits),Digits);
   if(ThisOrder.Type=="SELL") ThisOrder.Pips=DoubleToString(NormalizeDouble(PointsToPips(PriceMoveToPoints(OrderOpenPrice()-OrderClosePrice(),ThisOrder.Point)),Digits),Digits);   
   ThisOrder.A4="";     
   ThisOrder.Profit=DoubleToString(OrderProfit(),3); // without swaps and commissions 
   ThisOrder.PriceMove=DoubleToString(NormalizeDouble(OrderClosePrice()-OrderOpenPrice(),Digits),Digits);     
   ThisOrder.Commission=DoubleToString(OrderCommission(),2);
   ThisOrder.Swap=DoubleToString(OrderSwap(),Digits);   
   ThisOrder.MagicNumber=OrderMagicNumber();
   ThisOrder.Comment=CleanString(OrderComment()); 
   ThisOrder.A5="";  
   ThisOrder.A6="";          
   ThisOrder.StopLoss=DoubleToString(OrderStopLoss(),Digits);
   ThisOrder.TakeProfit=DoubleToString(OrderTakeProfit(),Digits);
return;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool SaveData(Orders &ThisOrder)
  {
//Save it to disk...
   string CleanComment;

   if(NeedToOpenFile)
     {
      ResetLastError();
      // filehandle=FileOpen(reportfilename,FILE_COMMON|FILE_WRITE|FILE_CSV); // overwrite previous file if present.  If written to the COMMON files directory, the SendFTP function will not work.
      filehandle=FileOpen(reportfilename,FILE_WRITE|FILE_CSV); // overwrite previous file if present.  Send to local files directory.  SendFTP should work.
      if(filehandle==INVALID_HANDLE || filehandle<=0) return(false); // we can't open the file so exit the try.
     }
   if(filehandle!=INVALID_HANDLE)
     {
      if(NeedToOpenFile) // write a header line first.
        {
         FileWrite(filehandle,
                   "AccNumber",
                   "SeqNum",
                   "Ticket",
                   "Type", 
                   "Lots",
                   "Symbol",
                   "ServerOpenTime_MT", 
                   "",
                   "ServerCloseTime_MT", 
                   "",
                   "",                   
                   "OpenPrice",                                                                                                                                                                        
                   "ClosePrice",
                   "Pips",
                   "",                      
                   "Profit",
                   "PriceMove",                                                       
                   "Commission",
                   "Swap",                   
                   "MagicNumber",
                   "Comment",   
                   "", 
                   "",                                                       
                   "StopLoss",
                   "TakeProfit");
         NeedToOpenFile=false; // Dont need to open it or write header line again
        }

      FileWrite(filehandle,
                ThisOrder.AccNumber,
                ThisOrder.SequenceNumber,
                ThisOrder.Ticket,
                ThisOrder.Type, 
                ThisOrder.Lots, 
                ThisOrder.Symbol, 
                ThisOrder.ServerOpenTime_MT, 
                ThisOrder.A1,
                ThisOrder.ServerCloseTime_MT,  
                ThisOrder.A2,
                ThisOrder.A3,                
                ThisOrder.OpenPrice,                                                                                                                        
                ThisOrder.ClosePrice,
                ThisOrder.Pips,
                ThisOrder.A4,                  
                ThisOrder.Profit,
                ThisOrder.PriceMove,                                              
                ThisOrder.Commission,
                ThisOrder.Swap,                
                ThisOrder.MagicNumber,
                ThisOrder.Comment,   
                ThisOrder.A5,  
                ThisOrder.A6,                                               
                ThisOrder.StopLoss,
                ThisOrder.TakeProfit);
      return(true);
     }
   else
     {
      MessageBox(StringConcatenate("Operation FileOpen failed for ",reportfilename,", error ",GetLastError()));
      return(false);
     }
   return(false);
  }
//+-----------------------------------------------------------------------------+
//| Make string safe for .CSV files using comma, tab or semi-colon as seperator |
//+-----------------------------------------------------------------------------+
string CleanString(string s)
  {
   string temp=s;
   StringReplace(temp,";","|"); // replace any ';' with '|' to avoid the field delimiter in the .csv file
   StringReplace(temp,",","|"); // replace any ',' with '|' to avoid the field delimiter in the .csv file
   StringReplace(temp,"\t"," "); // replace any {tab} with {space} to avoid the field delimiter in the .csv file
   return(temp);
  }
//+------------------------------------------------------------------+
//| Text name of order type                                          |
//+------------------------------------------------------------------+
string OrderTypeName(int TradeOperation)
  {
   switch(TradeOperation)
     {
      case 0 : return("BUY");
      case 1 : return("SELL");
      case 2 : return("BUYLIMIT");
      case 3 : return("SELLLIMIT");
      case 4 : return("BUYSTOP");
      case 5 : return("SELLSTOP");
      default : return("other");
     }
   return("unknown?");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string CleanDateTime(datetime d)
  {
   return(StringConcatenate(CleanDate(d)," ",CleanTime(d)));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string CleanDate(datetime d)
  {
   string temp;

   temp=StringConcatenate("00",IntegerToString(TimeYear(d)));
   string yyyy=StringSubstr(temp,(StringLen(temp)-4),4);

   temp=StringConcatenate("00",IntegerToString(TimeMonth(d)));
   string mm=StringSubstr(temp,(StringLen(temp)-2),2);

   temp=StringConcatenate("00",IntegerToString(TimeDay(d)));
   string dd=StringSubstr(temp,(StringLen(temp)-2),2);

   return(StringConcatenate(dd,"-",mm,"-",yyyy));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string CleanTime(datetime d)
  {
   string temp;

   temp=StringConcatenate("00",IntegerToString(TimeHour(d)));
   string hh=StringSubstr(temp,(StringLen(temp)-2),2);

   temp=StringConcatenate("00",IntegerToString(TimeMinute(d)));
   string mi=StringSubstr(temp,(StringLen(temp)-2),2);

   temp=StringConcatenate("00",IntegerToString(TimeSeconds(d)));
   string ss=StringSubstr(temp,(StringLen(temp)-2),2);

   return(StringConcatenate(hh,":",mi,":",ss));
  }
//+------------------------------------------------------------------+
//| Translate a price move (Buy-Sell) into points for the currency   |
//+------------------------------------------------------------------+
double PriceMoveToPoints(double pricemove,double point)
  {
   double p=point;
   if(p==0) p=SymbolInfoDouble(Symbol(),SYMBOL_POINT);

   if(Digits==3 || Digits==5) return((pricemove/p)*1);
   return((pricemove/p)*10);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double PointsToPips(double points)
  {
   if(Digits==3 || Digits==5) return(points/10);
   return(points);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double PipsToPoints(double pips)
  {
   if(Digits==3 || Digits==5) return(pips*10);
   return(pips);
  }
//+------------------------------------------------------------------+
//| Slot Day from GMT                                                |
//+------------------------------------------------------------------+
int SlotNumFromServerTime(datetime T=0)
  {
   if(T==0) T=TimeCurrent(); // last known server time
   int th,m;
   th=TimeHour(T+ServerTimeGMTOffsetSeconds); // hour number 0-23 on the server in GMT
   m=(int)MathFloor(TimeMinute(T+ServerTimeGMTOffsetSeconds)/(60/SLOTSPERHOUR));  // should yield 0-3 for 15 minute slots when SLOTSPERHOUR=4
   return((th*SLOTSPERHOUR)+m); // slot number of current GMT
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int SlotDayFromServerTime(datetime T=0)
  {
   if(T==0) T=TimeCurrent(); // last known server time
   TimeToStruct(T+ServerTimeGMTOffsetSeconds,ServerMqlDateTimeGMT);
   return(ServerMqlDateTimeGMT.day_of_week);
  }
//+------------------------------------------------------------------+
